import {
  Stack,
  TextField,
  Box,
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  FormHelperText,
  Button,
  Typography,
  CircularProgress,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
} from "@mui/material";
import SaveIcon from "@mui/icons-material/Save";
import { DataService } from "@/types";
import {
  Controller,
  FormProvider,
  SubmitHandler,
  useForm,
  useFormContext,
} from "react-hook-form";
import { ChangeEventHandler, useRef, useState } from "react";
import { useRouter } from "next/navigation";
import { LoadingButton } from "@mui/lab";
import { useHandleAuthErrors } from "@/hooks/useHandleAuthErrors";
import {
  useApiCreateDataService,
  useApiUpdateDataService,
} from "@/api/cubicweb";
import FileUploadIcon from "@mui/icons-material/FileUpload";
import CancelIcon from "@mui/icons-material/Cancel";
import { useUploadDataServiceFile } from "@/hooks/useUploadDataServiceFile";

export interface DataServiceFormProps {
  dataService?: DataService;
}

export function DataServiceForm({ dataService }: DataServiceFormProps) {
  const [loading, setLoading] = useState(false);
  const { handleSubmit, formState, reset, ...useFormProps } =
    useForm<DataService>({
      defaultValues: dataService
        ? dataService
        : {
            refresh_period: "weekly",
          },
    });
  const handleAuthErrors = useHandleAuthErrors();
  const createDataService = useApiCreateDataService();
  const updateDataService = useApiUpdateDataService();

  const router = useRouter();

  const onSubmit: SubmitHandler<DataService> = async (data) => {
    setLoading(true);
    try {
      if (data.eid === undefined) {
        const result = await createDataService(data);
        const eid = result[0][0];
        setLoading(false);
        router.push(`/data-service/${eid}`);
      } else {
        await updateDataService(data);
        reset(data);
        setLoading(false);
      }
    } catch (e) {
      handleAuthErrors(e);
    }
  };

  return (
    <FormProvider
      handleSubmit={handleSubmit}
      formState={formState}
      reset={reset}
      {...useFormProps}
    >
      <form onSubmit={handleSubmit(onSubmit)}>
        <Stack spacing={2}>
          {!dataService ? (
            <Controller
              name="name"
              rules={{ required: true }}
              render={({ field, fieldState: { error } }) => (
                <TextField
                  label="Nom"
                  required={true}
                  disabled={loading}
                  {...field}
                  error={error !== undefined}
                  helperText={
                    error?.type === "required" ? "Champ requis" : error?.message
                  }
                />
              )}
            />
          ) : null}
          <Stack direction={"row"} spacing={2}>
            <Box flex={1}>
              <Controller
                name="data_url"
                rules={{ required: true }}
                render={({ field, fieldState: { error } }) => (
                  <TextField
                    label="Data URL"
                    required={true}
                    disabled={loading}
                    {...field}
                    error={error !== undefined}
                    helperText={
                      error?.type === "required"
                        ? "Champ requis"
                        : error?.message
                    }
                    fullWidth
                  />
                )}
              />
            </Box>
            <GenerateLinkButton />
          </Stack>
          <Controller
            name="refresh_period"
            rules={{ required: true }}
            render={({ field, fieldState: { error } }) => (
              <FormControl fullWidth>
                <InputLabel id="update-freq-label">
                  Fréquence de mise à jour
                </InputLabel>
                <Select
                  labelId="update-freq-label"
                  label="Fréquence de mise à jour"
                  {...field}
                  error={error !== undefined}
                  disabled={loading}
                >
                  <MenuItem value={"daily"}>Journalier</MenuItem>
                  <MenuItem value={"weekly"}>Hebdomadaire</MenuItem>
                  <MenuItem value={"monthly"}>Mensuel</MenuItem>
                </Select>
                {error ? (
                  <FormHelperText error={true}>{error?.message}</FormHelperText>
                ) : null}
              </FormControl>
            )}
          />
          <Controller
            name="description"
            render={({ field, fieldState: { error } }) => (
              <TextField
                label="Description"
                disabled={loading}
                {...field}
                error={error !== undefined}
                helperText={error?.message}
                fullWidth
              />
            )}
          />
          <Box display={"flex"} flexDirection={"row-reverse"}>
            <LoadingButton
              variant="contained"
              startIcon={<SaveIcon />}
              loading={loading}
              type="submit"
              disabled={!formState.isDirty}
            >
              {dataService ? "Sauvegarder" : "Ajouter"}
            </LoadingButton>
          </Box>
        </Stack>
      </form>
    </FormProvider>
  );
}

function GenerateLinkButton() {
  const { setValue } = useFormContext();
  const ref = useRef<HTMLInputElement>(null);
  const { uploadFile, error, loading } = useUploadDataServiceFile();
  const [errorModalOpen, setErrorModalOpen] = useState(false);

  const chooseFile = () => {
    ref.current?.click();
  };

  const onFileChange: ChangeEventHandler<HTMLInputElement> = async (e) => {
    if (e.target.files && e.target.files.length > 0) {
      const file = e.target.files[0];
      const url = await uploadFile(file);
      if (url) {
        setValue("data_url", url, { shouldDirty: true });
      } else {
        setErrorModalOpen(true);
      }
    }
  };

  function closeErrorDialog() {
    setErrorModalOpen(false);
  }

  return (
    <>
      <input
        type="file"
        style={{ display: "none" }}
        ref={ref}
        onChange={onFileChange}
      />
      <Button
        variant="outlined"
        startIcon={<FileUploadIcon />}
        onClick={chooseFile}
      >
        à partir d&apos;un fichier
      </Button>
      <Dialog open={loading}>
        <DialogTitle>Mise en ligne du fichier...</DialogTitle>
        <DialogContent>
          <Box display={"flex"} alignItems={"center"} justifyContent={"center"}>
            <CircularProgress />
          </Box>
        </DialogContent>
      </Dialog>
      <Dialog open={errorModalOpen} onClose={closeErrorDialog}>
        <DialogTitle>Erreur de mise en ligne</DialogTitle>
        <DialogContent>
          <Typography>{error}</Typography>
          <Box display={"flex"} alignItems={"center"} justifyContent={"center"}>
            <CancelIcon color="error" fontSize="large" />
          </Box>
        </DialogContent>
        <DialogActions>
          <Button onClick={closeErrorDialog} autoFocus>
            Close
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}
